// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "ATTOAuthClient.h"
#import <AFHTTPClient.h>
#import <AFJSONRequestOperation.h>
#import "ATTAppConfiguration.h"
#import "OAuthToken.h"
#import "JSONParser.h"

NSString* const OAuthAuthorizationPath= @"oauth/v4/authorize";
NSString* const OAuthTokenPath= @"oauth/v4/token";

NSString* const AccessToken= @"AccessToken";
NSString* const ATRefreshToken= @"ATRefreshToken";
NSString* const ATType= @"ATType";
NSString* const ATExpirationDate= @"ATExpirationDate";

@implementation ATTOAuthClient{
    AFHTTPClient* _httpClient;
    ATTAppConfiguration* _appConfig;
}

#pragma mark Initialization 

-(id)initWithAppConfig:(ATTAppConfiguration*)appConfig{
    self= [super init];
    
    if (!self) {
        return nil;
    }
    
    _appConfig= appConfig;
    _httpClient = [[AFHTTPClient alloc] initWithBaseURL:_appConfig.baseURL];
    [_httpClient setDefaultHeader:@"Accept" value:@"application/json"];
    [_httpClient setDefaultHeader:@"Content-Type" value:@"application/x-www-form-urlencoded"];
    
    return self;
}

#pragma mark - Main Tasks

-(void)getTokenWithSuccessBlock:(void (^)(OAuthToken *))success
                        failure:(void (^)(NSError *))failure{
    
    AFHTTPRequestOperation* tokenOperation=
    [self createTokenOperationWithParams:[self paramsForCredentials]
                                 success:success
                                 failure:failure];
    // start the request
    [tokenOperation start];
}

-(void)getTokenWithCode:(NSString*)code
                                 success:(void (^)(OAuthToken*))success
                                 failure:(void (^)(NSError*))failure{
    
    AFHTTPRequestOperation* tokenOperation=
    [self createTokenOperationWithParams:[self paramsForAuthorizationCode:code]
                                 success:success
                                 failure:failure];
    // start the request
    [tokenOperation start];
}

-(void)getTokenWithRefreshToken:(NSString *)refreshToken
                                 success:(void (^)(OAuthToken*))success
                                 failure:(void (^)(NSError*))failure{

    AFHTTPRequestOperation* tokenOperation=
    [self createTokenOperationWithParams:[self paramsForRefreshToken:refreshToken]
                                 success:success
                                 failure:failure];
    
    // start the request
    [tokenOperation start];
}


#pragma mark - Token Request preparation

-(NSDictionary*)paramsForCredentials{
    // create the HTTP POST request for the token
    NSDictionary* params= @{@"client_id": _appConfig.key,
                            @"client_secret": _appConfig.secret,
                            @"grant_type": @"client_credentials",
                            @"scope": _appConfig.scope};
    
    return params;
}

-(NSDictionary*)paramsForAuthorizationCode:(NSString*)code{
    
    // create the HTTP POST request for the token
    NSDictionary* params= @{@"client_id": _appConfig.key,
                            @"client_secret": _appConfig.secret,
                            @"grant_type": @"authorization_code",
                            @"code": code};
    
    return params;
}

-(NSDictionary*)paramsForRefreshToken:(NSString*)refreshToken{
    
    // create the HTTP POST request for the token
    NSDictionary* params= @{@"client_id": _appConfig.key,
                            @"client_secret": _appConfig.secret,
                            @"grant_type": @"refresh_token",
                            @"refresh_token": refreshToken};
    return params;
}

-(AFHTTPRequestOperation*)createTokenOperationWithParams:(NSDictionary*)params
                                                 success:(void (^)(OAuthToken*))success
                                                 failure:(void (^)(NSError*))failure{
    
    if (!_httpClient) {
        NSLog(@"Must initialize _httpClient = (%@).", _httpClient);
        return nil;
    }
    
    NSMutableURLRequest* tokenRequest= [_httpClient requestWithMethod:@"POST"
                                                                 path:OAuthTokenPath
                                                           parameters:params];

    // Creat the AFJSONOperation for the token
    AFJSONRequestOperation* tokenOperation= [[AFJSONRequestOperation alloc] initWithRequest:tokenRequest];
    [tokenOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation, id responseObject) {
        // SUCCESS: We have a token
        // if the response is nil, or there's no `access_token` to be read
        if (!responseObject || !(responseObject)[@"access_token"]) {
            // report the error and return
            NSLog(@"Invalid response:%@", responseObject);
            return;
        }
        
        // Create an accessToken
         OAuthToken* token= [JSONParser getTokenFromJSON:responseObject];
        
        success(token);
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        // FAILURE: Didn't get the token
        NSMutableDictionary* info= [NSMutableDictionary dictionaryWithDictionary:error.userInfo];
        (info)[NSLocalizedDescriptionKey]= [NSString stringWithFormat:@"%@ - %@", operation.responseString, error.localizedDescription];
        NSError* customError= [NSError errorWithDomain:error.domain
                                                  code:error.code
                                              userInfo:info];
        failure(customError);
    }];
    
    return tokenOperation;
    
}


#pragma mark - Persisting credentials to Keychain

-(NSURLProtectionSpace*)currentProtectionSpace{
    // Initialize the protection space
    NSURLProtectionSpace* protectionSpace=
    [[NSURLProtectionSpace alloc] initWithHost:_httpClient.baseURL.host
                                          port:[_httpClient.baseURL.port intValue]
                                      protocol:_httpClient.baseURL.scheme
                                         realm:nil
                          authenticationMethod:NSURLAuthenticationMethodHTTPDigest];
    
    return protectionSpace;
}

-(void)saveCredentialsToKeychain{

    NSURLCredential* credential= [NSURLCredential credentialWithUser:_appConfig.key
                                                            password:_appConfig.secret
                                                         persistence:NSURLCredentialPersistencePermanent];
    
    [[NSURLCredentialStorage sharedCredentialStorage] setCredential:credential
                                                 forProtectionSpace:[self currentProtectionSpace]];
}

-(NSURLCredential*)currentCredentials{
    NSURLCredential *credential;
    NSDictionary *credentials;
    
    credentials = [[NSURLCredentialStorage sharedCredentialStorage]
                   credentialsForProtectionSpace:[self currentProtectionSpace]];
    
    credential = [credentials.objectEnumerator nextObject];
    
    return credential;
}

-(void)removeCurrentCredentialsFromKeyChain{
    
    [[NSURLCredentialStorage sharedCredentialStorage]
     removeCredential:[self currentCredentials]
     forProtectionSpace:[self currentProtectionSpace]];
    
}

#pragma mark - Caching the token

+(void)cacheToken:(OAuthToken*)token{
    
    NSUserDefaults* tokenDefault= [NSUserDefaults standardUserDefaults];
    
    if (!token.accessToken || !token.refreshToken
        || !token.type || !token.expirationDate) {
        NSLog(@"Error caching the token: %@", token);
        return;
    }
    
    [tokenDefault setObject:token.accessToken forKey:AccessToken];
    [tokenDefault setObject:token.type forKey:ATType];
    [tokenDefault setObject:token.expirationDate forKey:ATExpirationDate];
    [tokenDefault setObject:token.refreshToken forKey:ATRefreshToken];
    
}

+(OAuthToken*)getCachedToken{
    
    NSUserDefaults *tokenDefault= [NSUserDefaults standardUserDefaults];
    
    OAuthToken* token= [[OAuthToken alloc] init];
    token.accessToken= [tokenDefault objectForKey:AccessToken];
    token.refreshToken= [tokenDefault objectForKey:ATRefreshToken];
    token.type= [tokenDefault objectForKey:ATType];
    token.expirationDate= [tokenDefault objectForKey:ATExpirationDate];
    
    // if there's no token information
    if (!token.accessToken) {
        NSLog(@"Info: OAuthToken not cached");
        return nil;
    }
    
    // return the cached token
    return token;
}

+(void)clearCachedToken{
    NSUserDefaults* tokenDefault= [NSUserDefaults standardUserDefaults];
    
    [tokenDefault removeObjectForKey:AccessToken];
    [tokenDefault removeObjectForKey:ATRefreshToken];
    [tokenDefault removeObjectForKey:ATType];
    [tokenDefault removeObjectForKey:ATExpirationDate];
}


@end
