// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <AFHTTPClient.h>
#import <AFJSONRequestOperation.h>
#import <AFImageRequestOperation.h>
#import <AFNetworkActivityIndicatorManager.h>

#import "ATTRequest.h"
#import "ATTRequestManager.h"

#import "OAuthToken.h"
#import "ATTAppConfiguration.h"
#import "ATTOAuthClient.h"

NSString* const ATTRequestErrorKey= @"ATTErrorKey";
NSString* const ATTServiceException= @"ServiceException";
NSString* const ATTPolicyException= @"PolicyException";
NSString* const ATTExceptionID= @"MessageId";
NSString* const ATTExceptionText= @"Text";

@implementation ATTRequestManager

#pragma mark - Constructors

- (id)initWithConfig:(ATTAppConfiguration*)appConfig andDelegate:(id<ATTOAuthConsent>)delegate{
    
    _delegate= delegate;
    _appConfig= appConfig;
    
    // return the newly created ATTRequestManager
    return [self initWithBaseURL:_appConfig.baseURL];
}


- (id)initWithBaseURL:(NSURL*)baseURL{
    
    // if everything is ok, initialize the http client
    self= [super init];
    
    if (!self) {
        return nil;
    }
    
    // enable token caching by default
    _shouldCacheToken= YES;
    _httpClient= [[AFHTTPClient alloc] initWithBaseURL:baseURL];
    // all the request are JSON requests
    [_httpClient registerHTTPOperationClass:[AFJSONRequestOperation class]];
    // set parameter encoding to JSON. This will set the body parameters
    // encoding for HTTP Methods POST and UPDATE
    [_httpClient setParameterEncoding:AFJSONParameterEncoding];
    
    [[AFNetworkActivityIndicatorManager sharedManager] setEnabled:YES];
    
    return self;
}

/** This will update the `Authorization` header for all requests
 NOTE: Always use this property to update the token.
 */
-(void)setOauthToken:(OAuthToken *)oauthToken{
    
    if (!oauthToken) {
        _oauthToken= oauthToken;
        return;
    }
    
    // add the authorization token to all subsequent requests
    [_httpClient setDefaultHeader:@"Authorization"
                            value:[NSString stringWithFormat:@"Bearer %@", oauthToken.accessToken]];
    
    // cache the token
    if (_shouldCacheToken) {
        [ATTOAuthClient cacheToken:oauthToken];
    }
    
    _oauthToken= oauthToken;
}

#pragma mark - Making requests

- (AFHTTPRequestOperation*)createHTTPRequestOperationFrom:(ATTRequest *)attRequest
{
    // setup additional parameters that might be needed for the request
    
    
    // Create the actual HTTP Request
    NSMutableURLRequest* urlRequest= [_httpClient requestWithMethod:attRequest.method
                                                               path:attRequest.path
                                                         parameters:attRequest.params];
    
    if ([attRequest.method isEqualToString:@"PUT"]) {
        // TODO: Review Bug Fix: PUT requests fail if the charset is specified.
        [urlRequest setValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
    }
   
   [urlRequest setValue:@"ClientSdk=att.iam.ios.1.1" forHTTPHeaderField:@"X-Arg"];
    
    // create the request operation
    AFHTTPRequestOperation* requestOperation= nil;
    
    if ([attRequest.responseType isEqualToString:RTJson]) {
        requestOperation = [[AFJSONRequestOperation alloc] initWithRequest:urlRequest];
    }else{
        requestOperation = [[AFHTTPRequestOperation alloc] initWithRequest:urlRequest];
    }
    
    return requestOperation;
}

- (NSError *)createCustomErrorFrom:(AFHTTPRequestOperation *)operation error:(NSError *)error
{
    // create custom error details dictionary with the information about
    // the request operation that failed.
    NSMutableDictionary* userInfo= [NSMutableDictionary dictionaryWithDictionary:error.userInfo];
    
    // if the response is empty, don't add anything to the error
    if (!operation.response) {
        return error;
    }
    
    // if there response contains a JSON description, get it's dictionary representation
    NSDictionary* attInfo= nil;
    if ([operation.response.MIMEType isEqualToString:@"application/json"]) {
        NSError* serializationError= nil;
        // get the json string
        attInfo= [NSJSONSerialization JSONObjectWithData:
                                [operation.responseString dataUsingEncoding: NSUTF8StringEncoding]
                                                               options:NSJSONReadingMutableContainers
                                                                 error:&serializationError];
        
        if (serializationError) {
            NSLog(@"Error serializing string: %@", operation.responseString);
        }
    }
    
    // if available, add the specific error in JSON format
    if (attInfo) {
        
        NSDictionary* requestError= (attInfo)[@"RequestError"];
        NSDictionary* exceptionInfo= nil;
        
        if (requestError[ATTServiceException]) {
            exceptionInfo= @{ATTExceptionID: (requestError)[ATTServiceException][ATTExceptionID],
                             ATTExceptionText: (requestError)[ATTServiceException][ATTExceptionText]};
        }else if ((requestError)[ATTPolicyException]){
            exceptionInfo= @{ATTExceptionID: (requestError)[ATTPolicyException][ATTExceptionID],
                             ATTExceptionText: (requestError)[ATTPolicyException][ATTExceptionText]};
        }else{
            exceptionInfo= @{ATTExceptionID: @"NoID",
                             ATTExceptionText: @"Unknown Exception"};
        }
        
        (userInfo)[ATTRequestErrorKey]= exceptionInfo;
    
        NSString* extraInfo = [NSString stringWithFormat: @"### Error: %@ - %@",
                               (exceptionInfo)[ATTExceptionID],
                               (exceptionInfo)[ATTExceptionText]];
        
        // Set the actual description
        (userInfo)[NSLocalizedDescriptionKey]= [NSString stringWithFormat:
                                                @"%@ - %@", extraInfo, error.localizedDescription];
        
    }else{ // add the response string
        (userInfo)[ATTRequestErrorKey]= operation.responseString;
    }
    
    // Define a custom NSError and pass that to the user's error handling code
    NSError* customError= [NSError errorWithDomain:error.domain
                                              code:error.code
                                          userInfo:userInfo];
    return customError;
}


-(id)parsedObjectForSuccessfulOperation:(AFHTTPRequestOperation *)operation
                           withResponse:(id)response
                          forATTRequest:(ATTRequest *)attRequest
                handleParsingErrorsWith:(void (^)(NSError *))failure{
    
    NSError* parseError= nil;
    // create a native (Cocoa) representation for the reponse object
    id parsedObject= [attRequest parseResponse:response forOperation:operation
                                          error:&parseError];
    
    // handle error with user-defined failure callback
    if (parseError) {
        
        // create a custom error en propagate it upwards
        NSError* customError= [self createCustomErrorFrom:operation error:parseError];
        
        // return early if there's no failure callback defined
        if (!failure) {
            NSLog(@"No failure block defined to process Error: %@", customError);
            return nil;
        }
        
        // bubble the error up
        failure(customError);
    }
    
    return parsedObject;
}

-(AFHTTPRequestOperation*)prepareHTTPRequestOperation:(ATTRequest*)attRequest
                                              success:(void (^)(id))success
                                              failure:(void (^)(NSError *))failure{
    
    // The request resource path should not be the empty string
    if (0 == [attRequest.path length]) {
        // report this error
        NSString* description= [NSString stringWithFormat:
                                @"Invalid resource path: \"%@\"", attRequest.path];
        NSError* error= [NSError errorWithDomain:@"RequestValidationDomain"
                                            code:100
                                        userInfo:@{NSLocalizedDescriptionKey: description}];
        if (failure) {
            failure(error);
        }
        // return early, can't create a valid request
        return nil;
    }
    
    // create the actual URL request
    AFHTTPRequestOperation* requestOperation = [self createHTTPRequestOperationFrom:attRequest];
    
    // setup the HTTPOperation succes and failure callbacks
    [requestOperation setCompletionBlockWithSuccess:^(AFHTTPRequestOperation *operation,
                                                      id response) {
        
        // return early if success callback is not defined.
        // no use going through all the trouble if nothig is to be done
        // with the result
        if (!success) {
            NSLog(@"No success block defined: Nothing to with response: %@",
                  [response description]);
            return;
        }
        
        // Try parsing the response
        id parsedObject= [self parsedObjectForSuccessfulOperation:operation
                                                     withResponse:response
                                                    forATTRequest:attRequest
                                          handleParsingErrorsWith:failure];
        
        // return early if the response couln't be parsed
        if (!parsedObject) {
            return;
        }
        
        // there were no parsing errors
        // pass the parsed object up to be processed
        // with user-provided callback
        success(parsedObject);
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        
        // detect whether this is due to an expired/invalid Access Token (AT)
        // 401 means that the AT is expired or that it has been
        // invalidated.
        // NOTE: If we receive this error code is because we tried using an
        // invalid/expired AT, so it's safe to assume we have a
        // refresh token.
        if (401 == operation.response.statusCode) {
            // print error description
            NSError *customError = [self createCustomErrorFrom:operation error:error];
            NSLog(@"Error: %@ \nWill try to get a new one...", customError.localizedDescription);
            
            // will try to get a new token, and then reprocess the request
            [self getNewTokenAndProcess:attRequest success:success failure:failure];
            return;
        }
        
        // If that doesn't work, you'll have to ask the user to authorize
        // the app again, do so by sending the `showAuthorizationPage` message
        // to the delegate.
        
        NSError *customError = [self createCustomErrorFrom:operation error:error];
        
        // report error & return early if failure callback is not defined
        if (nil == failure) {
            NSLog(@"No failure block defined to process Error: %@",
                  customError.localizedDescription);
            return;
        }
        
        // pass the error up
        failure(customError);
    }];
    
    return requestOperation;
}

-(void)sendAsynchronous:(ATTRequest *)attRequest
                success:(void (^)(id result))success
                failure:(void (^)(NSError *error))failure{
    
    // If we don't have a token
    if (!_oauthToken) {
        // try to get the access token from cache
        _oauthToken= [ATTOAuthClient getCachedToken];
    }
    
    // If there's a token but it's expired
    if (_oauthToken && [_oauthToken isExpired]) {
        
        // get a new one using the refresh token.
        ATTOAuthClient* oauthClient= [[ATTOAuthClient alloc] initWithAppConfig:_appConfig];
        [oauthClient getTokenWithRefreshToken:_oauthToken.refreshToken
                                      success:^(OAuthToken *token) {
                                          
                                          // we have a new token, so update the current token
                                          [self setOauthToken:token];
                                          // try sending the last request with the new token
                                          [self sendAsynchronous:attRequest success:success failure:failure];
                                          
                                      } failure:^(NSError *error) {
                                          // we don't have a new token, let's report that as an error.
                                          NSLog(@"Error trying to get a new token: %@", error);
                                          failure(error);
                                      }];
        
        // we can't do anything, until we get the new token
        return;
    }

    // if we still don't have a token, tell the delegate
    // if by this point we don't have a token, it means one of the following:
    // - it's not in cache
    // - it is in cache, but it's expired
    if (!_oauthToken) {
        // in that case we cannot get the token from the library, the delegate has
        // to go and get a new token by using the OAuth consent flow, i.e. by
        // trying to load the OAuth Authoirzation Page.
        [_delegate showAuthorizationPage];
        
        // we can't do anything without the token, ....
        return;
    }
    
    
    // on the other hand, if we do have a valid token...
    // make sure the token is in the header
    [self setOauthToken:_oauthToken];
    
    // create the HTTP Request operation from the ATTRequest object
    AFHTTPRequestOperation* requestOperation= [self prepareHTTPRequestOperation:attRequest
                                                                        success:success
                                                                        failure:failure];
    // return early if the operation could not be created
    if (!requestOperation) {
        NSLog(@"Error while creating AFHTTPOperation for ATTRequest: %@", [attRequest description]);
        return;
    }
    
    // execute the request operation
    [requestOperation start];
    
}


#pragma mark - Request Access Token if Invalid/Expired

-(void)getNewTokenAndProcess:(ATTRequest*)attRequest
                     success:(void (^)(id))success
                     failure:(void (^)(NSError *))failure{
    
    // try to get a new token using the refresh token
    __weak id weakSelf= self; // we don't want a retain cycle ergo use `__weak`
    [self requestNewTokenWithSuccessBlock:^(OAuthToken* token) {
        // update the current token
        _oauthToken= token;
        // try sending the original request
        [weakSelf sendAsynchronous:attRequest success:success failure:failure];
    } failure:^(NSError* error) {
        NSLog(@"Failed to get a new token:%@", error.localizedDescription);
        // pass the error up to be processed.
        failure(error);
    }];
}

/** This will try to get a new token using the current refresh token.
    It will try to process it like any other ATTRequest.
 */
-(void)requestNewTokenWithSuccessBlock:(void (^)(OAuthToken*))success
                               failure:(void (^)(NSError*))failure{
    
    ATTOAuthClient* oauth= [[ATTOAuthClient alloc] initWithAppConfig:_appConfig];

    [oauth getTokenWithRefreshToken:_oauthToken.refreshToken
                                             success:success
                                             failure:failure];
}

@end
