// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "JSONParser.h"


#import "ATTRequest.h"
#import "IAMMessageRequest.h"
#import "IAMMessagesListRequest.h"

#import "IAMMessage.h"
#import "IAMQueue.h"
#import "TypeMetaData.h"
#import "MMSContent.h"
#import "SegmentationDetails.h"
#import "IAMDelta.h"
#import "IAMDeltaDetail.h"
#import "IAMMessageIndexInfo.h"
#import "IAMNotificationConnection.h"
#import "OAuthToken.h"

#import "NSString+IAMUtils.h"

NSString* const LongDateFormat= @"yyyy-MM-dd'T'HH:mm:ss";

@implementation JSONParser


+(OAuthToken*)getTokenFromJSON:(id)tokenJSON{
    // Create an accessToken
    OAuthToken* token= [[OAuthToken alloc] init];
    token.accessToken= (tokenJSON)[@"access_token"];
    token.type= (tokenJSON)[@"token_type"];
    NSDate* expirationDate= [NSDate dateWithTimeIntervalSinceNow:
                             [(tokenJSON)[@"expires_in"] intValue]];
    token.expirationDate= expirationDate;
    token.refreshToken= (tokenJSON)[@"refresh_token"];
    
    return token;
}
+ (IAMMessage*)getMessageFromJSON:(id)messageJSON{
    
    // setup formatters for dates and numbers
    NSDateFormatter* df= [[NSDateFormatter alloc] init];
    df.dateFormat= LongDateFormat;
    NSNumberFormatter* nf= [[NSNumberFormatter alloc] init];
    nf.numberStyle= NSNumberFormatterDecimalStyle;
    
    IAMMessage* iamMessage= [[IAMMessage alloc] init];
    
    iamMessage.messageId= (messageJSON)[@"messageId"];
    iamMessage.from= (messageJSON)[@"from"][@"value"];
    
    NSMutableArray* mRecipients= [NSMutableArray array];
    for (NSDictionary* recipient in (messageJSON)[@"recipients"]) {
        [mRecipients addObject:(recipient)[@"value"]];
    }
    iamMessage.recipients= mRecipients;
    
    iamMessage.date= [(messageJSON)[@"timeStamp"] dateWithFormatter:df];
    
    iamMessage.text= (messageJSON)[@"text"];
    
    iamMessage.favorite= [(messageJSON)[@"isFavorite"] boolValue];
    iamMessage.unread= [(messageJSON)[@"isUnread"] boolValue];
    
    iamMessage.type= [(messageJSON)[@"type"] intValue];
    
    NSDictionary* metaData= (messageJSON)[@"typeMetaData"];
    // only parse meta-data if it is available
    if (metaData) {
        
        iamMessage.typeMetaData= [[TypeMetaData alloc] init];
        iamMessage.typeMetaData.subject= (messageJSON)[@"typeMetaData"][@"subject"];
        iamMessage.typeMetaData.segmented= [(messageJSON)[@"typeMetaData"][@"subject"] boolValue];
        
        NSDictionary* segDetails= (metaData)[@"segmentationDetails"];
        // only parse the segmentation details if they are available
        if (segDetails) {
            NSInteger reference= [(segDetails)[@"segmentationMsgRefNumber"] intValue];
            NSInteger part= [(segDetails)[@"thisPartNumber"] intValue];
            NSInteger totalParts= [(segDetails)[@"totalNumberOfParts"] intValue];
            
            iamMessage.typeMetaData.details= [[SegmentationDetails alloc]init];
            iamMessage.typeMetaData.details.reference= reference;
            iamMessage.typeMetaData.details.part= part;
            iamMessage.typeMetaData.details.totalParts= totalParts;
            
        }
    }
    
    iamMessage.incoming= [(messageJSON)[@"isIncoming"] boolValue];
    
    // Get the MMS Content array
    NSArray* mms= (messageJSON)[@"mmsContent"];
    // only parse the MMS content if is not empty
    if (mms && [mms count]>0) {
        
        // create a native MMSContent array
        NSMutableArray* mContentArray= [NSMutableArray array];
        for (NSDictionary* content in mms) {
            MMSContent* currentMMS= [[MMSContent alloc] init];
            currentMMS.contentType = (content)[@"contentType"];
            currentMMS.name= (content)[@"contentName"];
            currentMMS.path= (content)[@"contentUrl"];
            currentMMS.type= [(content)[@"type"] getMMSType];
            
            [mContentArray addObject:currentMMS];
        }
        
        iamMessage.mmsContent= mContentArray;
    }
    
    return iamMessage;
}

+(NSArray *)getMessageListFromJSON:(NSDictionary *)json{
    
    NSArray* messages= (json)[@"messageList"][@"messages"];
    
    // return emtpy array in case the messages key is not defined
    // or is an emtpy array.
    if (!messages || [messages count]==0) {
        return @[];
    }
    
    // By this point we are certain that the messages array is not emtpy
    
    // initialize temporary array for storing the parsed messages
    NSMutableArray* nativeMessages= [NSMutableArray array];
    
    //  create a native IAMMessage for each element in the JSON array
    for (id messageJSON in messages) {
        
        IAMMessage* iamMessage= [JSONParser getMessageFromJSON:messageJSON];
        
        if (!iamMessage) {// skip nil messages
            continue;
        }
        
        [nativeMessages addObject:iamMessage];
    }
    
    // if there are no messages, an emtpy array is returned.
    return nativeMessages;
}

+(NSArray*)getDeltaDetailsFromJSON:(NSDictionary*)deltaJSON{
    
    
    if (0 == [deltaJSON count]) {
        return @[];
    }
    
    NSMutableArray* details= [NSMutableArray array];
    for (NSDictionary* currentMessageJSON in deltaJSON) {
        
        IAMDeltaDetail* currentDetails= [[IAMDeltaDetail alloc] init];
        currentDetails.messageId= (currentMessageJSON)[@"messageId"];
        currentDetails.favorite= [(currentMessageJSON)[@"isFavorite"] boolValue];
        currentDetails.unread= [(currentMessageJSON)[@"isUnread"] boolValue];
        
        [details addObject:currentDetails];
    }
    
    return details;
}

+(NSArray *)getDeltasFromJSON:(id)deltasJSON{
    
    if (0 == [deltasJSON count]) {
        return @[];
    }
    
    NSMutableArray* deltas= [NSMutableArray array];
    
    for (NSDictionary* currentDeltaJSON in deltasJSON) {
        
        IAMDelta* currentDelta= [[IAMDelta alloc] init];
        currentDelta.type = [(currentDeltaJSON)[@"type"] getMessageType];
        
        id additionsJSON= (currentDeltaJSON)[@"adds"];
        if (additionsJSON) {
            currentDelta.additions= [JSONParser getDeltaDetailsFromJSON:additionsJSON];
        }
        
        id deletionsJSON= (currentDeltaJSON)[@"deletes"];
        if (deletionsJSON) {
            currentDelta.deletions= [JSONParser getDeltaDetailsFromJSON:deletionsJSON];
        }
        
        id updatesJSON= (currentDeltaJSON)[@"updates"];
        if (updatesJSON) {
            currentDelta.updates= [JSONParser getDeltaDetailsFromJSON:updatesJSON];
        }
        
        [deltas addObject:currentDelta];
    }
    
    return deltas;
}


+(id)getMessageIndexInfoFromJSON:(id)indexInfoJSON{
    
    IAMMessageIndexInfo* indexInfo= [[IAMMessageIndexInfo alloc] init];
    
    indexInfo.indexCacheStatus = [IAMMessageIndexInfo statusFromString: (indexInfoJSON)[@"status"]];
    indexInfo.mailboxState= (indexInfoJSON)[@"state"];
    indexInfo.cachedMessageCount= [(indexInfoJSON)[@"messageCount"] intValue];
    
    return indexInfo;
}

+(id)getConnectionFromJSON:(id)connectionJSON{
    
    IAMNotificationConnection* connection= [[IAMNotificationConnection alloc] init];
    
    NSDictionary* details= (connectionJSON)[@"notificationConnectionDetails"];
    
    if (!details) {
        // error
        return nil;
    }
    
    // map the details of the connection
    connection.userName= (details)[@"username"];
    connection.password= (details)[@"password"];
    connection.httpsURL= (details)[@"httpsUrl"];
    connection.webSocketURL= (details)[@"wssUrl"];
    
    NSMutableArray* queues= [NSMutableArray array];
    
    NSDictionary* queuesDic= (details)[@"queues"];
    
    for (NSString* key in [queuesDic allKeys]) {
        IAMQueue* currentQueue= [[IAMQueue alloc] init];
        
        if (NSOrderedSame == [key caseInsensitiveCompare:@"text"]) {
            currentQueue.type= IAMTextQueue;
        }
        if (NSOrderedSame == [key caseInsensitiveCompare:@"mms"]) {
            currentQueue.type= IAMMMSQueue;
        }else{
            currentQueue.type= IAMUnknownQueue;
        }
        currentQueue.path= (queuesDic)[key];
        
        
        [queues addObject:currentQueue];
    }
    
    connection.queues= queues;
    
    return connection;
}
@end
