// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "MyCustomXCTestCase.h"
#import "ATTAppConfiguration.h"
#import "ATTOAuthClient.h"
#import "OAuthToken.h"

@interface TestATTOAuthClient : MyCustomXCTestCase

@end

@implementation TestATTOAuthClient{
    ATTAppConfiguration* _appConfig;
    ATTOAuthClient* oauth;
    OAuthToken* tokenToCache;
}

- (void)setUp
{
    [super setUp];
    // initialize the app configuration
    _appConfig= [[ATTAppConfiguration alloc] init];
    _appConfig.key= @"youlfuynskbdwzdybyz4pjk1q2rhvipb";
    _appConfig.secret= @"tgihknj57jeqvcafeglaqfdvnqpywhgc";
    // setting the initial scope
    _appConfig.scope= @"MIM,IMMN";
    _appConfig.baseURL= [NSURL URLWithString:@"https://api.att.com"];
    _appConfig.redirectURL= [NSURL URLWithString:@"http://localhost"];
    
    oauth= [[ATTOAuthClient alloc] initWithAppConfig:_appConfig];
    
    // init dummy token, used to test caching methods, thus they are random strings
    tokenToCache= [[OAuthToken alloc] init];
    tokenToCache.accessToken= @"at";
    tokenToCache.refreshToken= @"rt";
    tokenToCache.expirationDate= [NSDate dateWithTimeIntervalSinceNow:1000000];
    tokenToCache.type= @"type";
}

- (void)tearDown
{
    // Put teardown code here; it will be run once, after the last test case.
    [super tearDown];
}

/** This test can get a token only for API's (Scopes) that DON'T require
    user consent. It will fail if you try to use it with User Consent-based 
    scopes. 
 
    Make sure your application configuration in the Developer Portal matches 
    with `_appConfig` properties */
- (void)testGetToken{

    // This type of token request only applies to the following scopes
    _appConfig.scope= @"ADS,PAYMENT,SMS";
    
    [oauth getTokenWithSuccessBlock:^(OAuthToken *token) {
        
        XCTAssert(nil != token, @"Error getting the token");
        NSLog(@"AT: %@", token);
        
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:5];
    
}


/** NOTE: You have to use a new code everytime you run this test.
    You can generate one by pasting this url in your browser and 
    allowing the application to send messages in your phone's behalf.
 
    https://api.att.com/oauth/authorize?client_id=7rxkasvlp0u3vxcwrbqyxmrtydjj3gtu&scope=IMMN,MIM&redirect_uri=http://localhost
 */
-(void)testGetTokenWithCode{
    
    
    // Applies only to scopes `DC,MIM,IMMN`
    _appConfig.scope= @"IMMN,MIM";
    [oauth getTokenWithCode:@"tcLLRLyllGUH7WHHEQOv"
                    success:^(OAuthToken *token) {
                        XCTAssert(nil != token, @"Error getting the token");
                        NSLog(@"AT: %@", token);
                        
                    } failure:^(NSError *error) {
                        XCTFail(@"%@", error.localizedDescription);
                    }];
    
    [self waitForMe:5];
}

/** NOTE: You have to use a new refresh token every time you run this test.
    The refresh token becomes invalid after using it to get a new token.
    Instead use the refresh token generated from the last time this test was run
    or by getting a new token using the test: `testGetTokenWithCode`
 */

- (void)testGetTokenWithRefreshToken
{
    _appConfig.scope= @"IMMN,MIM";
    [oauth getTokenWithRefreshToken:@"WNr8WS8kcikEqVS55EeN7YjoUUZxZjd0"
                                             success:^(OAuthToken *token) {
                                                 XCTAssert(nil != token, @"Error getting the token");
                                                 NSLog(@"AT: %@", token);
                                                 
                                             } failure:^(NSError *error) {
                                                 XCTFail(@"%@", error.localizedDescription);
                                             }];
    
    [self waitForMe:5];
}

#pragma mark - Persisting credentials to the Keychain - http://stackoverflow.com/a/17997943/400544

-(void)testGetSetCredentials{

    [oauth saveCredentialsToKeychain];
    
    NSURLCredential* credentials= [oauth currentCredentials];
    XCTAssertNotNil(credentials, @"Couldn't get current credentials.");
    NSLog(@"The credentials are: \nAppKey: %@\nAppSecret: %@", credentials.user, credentials.password);
}

#pragma mark - Token Cache

-(void)testCacheToken{
    
    [ATTOAuthClient cacheToken:tokenToCache];
    
    XCTAssertNotNil([ATTOAuthClient getCachedToken], @"Error there's NO cached token.");
    
}

-(void)testRemoveCachedToken{
    [ATTOAuthClient cacheToken:tokenToCache];
    
    [ATTOAuthClient clearCachedToken];
    
    OAuthToken* cachedToken= [ATTOAuthClient getCachedToken];
    XCTAssertNil(cachedToken, @"Error, there's is a cached token: %@", cachedToken);
}

@end
