// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "ViewController.h"
#import "ATTRequestManager.h"
#import "IAMManager.h"
#import "IAMMessage.h"
#import "TypeMetaData.h"
#import "ATTAppConfiguration.h"

#import "ATTOAuthViewController.h"

NSString* const APIBaseURL=@"https://api.att.com";
NSString* const AppScope= @"MIM";

// insert your app's values as registered on the ATT developer web site
//   NSString* const RedirectURL= @"https://example.com";
//   NSString* const AppKey= @"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
//   NSString* const AppSecret= @"
NSString* const RedirectURL= @ATT_APP_REDIRECT_URL;
NSString* const AppKey= @ATT_APP_KEY;
NSString* const AppSecret= @ATT_APP_SECRET;

@interface ViewController ()

@end

@implementation ViewController{
    IAMManager* _iamManager;
    ATTOAuthViewController* _oauthVC;
    __block NSArray* _messages;
    ATTAppConfiguration* _appConfig;
}

#pragma mark - Initialization

-(void)viewDidLoad{
    
    // initialize the app configuration
    _appConfig= [[ATTAppConfiguration alloc] init];
    _appConfig.key= AppKey;
    _appConfig.secret= AppSecret;
    _appConfig.scope= AppScope;
    _appConfig.baseURL= [NSURL URLWithString:APIBaseURL];
    _appConfig.redirectURL= [NSURL URLWithString:RedirectURL];

    [self initIAM];
    
    // to update the table view
    [self.tableView setDataSource:self];
    
}

-(void)initIAM{
    
    // if _iamManager is NOT initialized
    if (!_iamManager) {
        // initialize an IAMManager, it will try to use existing access token
        // but if it cannot find one, it will send message `showAuthorizationPage`
        // to it's delegate.
        
        _iamManager= [[IAMManager alloc] initWithConfig:_appConfig andDelegate:self];
    }
    
    // if _iamManager is still nil, it means there' was an error initializing it.
    if (!_iamManager) {
        UIAlertView* errorView= [[UIAlertView alloc] initWithTitle:@"Error"
                                                           message:@"Could NOT initilize the request manger"
                                                          delegate:self
                                                 cancelButtonTitle:@"OK"
                                                 otherButtonTitles:nil];
        [errorView show];
        
        return;
    }
    
    // if everything went ok, we should have a valid instance for _iamManager
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - UI Events


-(IBAction)doSomethingWithIAM:(id)sender{
    [self getAllMessages];
}


-(void)getAllMessages{
    // Start doing something with the `iamManager`
    
    [_iamManager getAllMessages:^(NSArray* messages) {
        NSLog(@"%@", messages);
        
        _messages= messages;
        [self.tableView reloadData];
        
    } failure:^(NSError *error) {
        NSLog(@"Couln't get messages, try again: %@",error.localizedDescription);
    }];
}

#pragma mark - ATTOAuthConsent

-(void)didCancel{
    if (![self.presentedViewController isBeingDismissed]) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

-(void)showAuthorizationPage{

    _oauthVC= [[ATTOAuthViewController alloc] init];
    _oauthVC.delegate= self;
    [_oauthVC setConfiguration:_appConfig];
    
    [self presentViewController:_oauthVC animated:YES completion:nil];
}


-(void)didGetToken:(OAuthToken *)token{
    
    // use the token just received
    _iamManager.oauthToken= token;
    if (![self.presentedViewController isBeingDismissed]) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
    [self getAllMessages];
}

-(void)didFailToObtainToken:(NSError *)error{
    
    NSLog(@"%@", error);
    if (![self.presentedViewController isBeingDismissed]) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}


#pragma mark - UITableDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return ((!_messages)? 0: _messages.count);
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    // Configure the cell...
    IAMMessage* message= _messages[indexPath.row];
    if (MTMMS == message.type) {
        cell.textLabel.text= message.typeMetaData.subject;
    }else{
        cell.textLabel.text= message.text;
    }
    cell.detailTextLabel.text= message.from;
    
    return cell;
}



@end
